@@ -5,7 +5,7 @@ Scanning is a general form of recurrence, which can be used for looping.
 The idea is that you *scan* a function along some input sequence, producing
 an output at each time-step that can be seen (but not modified) by the
 function at the next time-step. (Technically, the function can see the
-previous K  time-steps of your outputs and L time steps (from the past and
+previous K  time-steps of your outputs and L time steps (from past and
 future) of your inputs.
 
 So for example, ``sum()`` could be computed by scanning the ``z+x_i``
@@ -13,15 +13,21 @@ function over a list, given an initial state of ``z=0``.
 
 Special cases:
 
-* A *reduce* operation can be performed by returning only the last
+* A *reduce* operation can be performed by using only the last
   output of a ``scan``.
 * A *map* operation can be performed by applying a function that
   ignores previous steps of the outputs.
 
-Often a for-loop can be expressed as a ``scan()`` operation, and ``scan`` is
-the closest that theano comes to looping. The advantage of using ``scan``
-over for loops is that it allows the number of iterations to be a part of
-the symbolic graph.
+Often a for-loop or while-loop can be expressed as a ``scan()`` operation,
+and ``scan`` is the closest that theano comes to looping. The advantages
+of using ``scan`` over `for` loops in python (amongs other) are:
+
+* it allows the number of iterations to be part of the symbolic graph
+* it allows computing gradients through the for loop
+* there exist a bunch of optimizations that help re-write your loop
+such that less memory is used and that it runs faster
+* it ensures that data is not copied from host to gpu and gpu to
+host at each step
 
 The Scan Op should typically be used by calling any of the following
 functions: ``scan()``, ``map()``, ``reduce()``, ``foldl()``,
@@ -39,7 +45,6 @@ __contact__ = "Razvan Pascanu <r.pascanu@gmail>"
 import itertools
 import logging
 import numpy
-import warnings
 
 from theano.compile import SharedVariable, function
 from theano import compile
@@ -47,21 +52,15 @@ from theano import gof
 from theano.tensor import opt
 from theano import tensor
 from theano import config
-
-import scan_op
-from scan_op import safe_new, safe_to_cpu
-import scan_utils
-from scan_utils import safe_new, safe_to_cpu, traverse
+from theano.updates import Updates
 from theano.sandbox import cuda
 
+import scan_op
+import scan_utils
+from scan_utils import safe_new, traverse
+
 # Logging function for sending warning or info
-_logger = logging.getLogger('theano.scan')
-
-def warning(*msg):
-    _logger.warning('WARNING theano.scan: '+' '.join(msg))
-
-def info(*msg):
-    _logger.info('INFO theano.scan: '+' '.join(msg))
+_logger = logging.getLogger('theano.scan_module.scan')
 
 
 def scan( fn
@@ -72,7 +71,8 @@ def scan( fn
          , truncate_gradient = -1
          , go_backwards      = False
          , mode              = None
-         , name              = None ):
+         , name              = None
+         , profile           = False):
     """
     This function constructs and applies a Scan op to the provided
     arguments.
@@ -81,27 +81,27 @@ def scan( fn
         ``fn`` is a function that describes the operations involved in one
         step of ``scan``. ``fn`` should construct variables describing the
         output of one iteration step. It should expect as input theano
-        variables representing all the time slices of the input sequences
-        and outputs, and all other arguments given to scan as
-        ``non_sequences``. The order in which scan passes this variables
-        to ``fn``  is the following :
+        variables representing all the slices of the input sequences
+        and previous values of the outputs, as well as all other arguments
+        given to scan as ``non_sequences``. The order in which scan passes
+        these variables to ``fn``  is the following :
 
         * all time slices of the first sequence
         * all time slices of the second sequence
         * ...
         * all time slices of the last sequence
-        * all time slices of the first output
-        * all time slices of the second otuput
+        * all past slices of the first output
+        * all past slices of the second otuput
         * ...
-        * all time slices of the last output
+        * all past slices of the last output
         * all other arguments (the list given as `non_sequences` to
             scan)
 
         The order of the sequences is the same as the one in the list
-        `sequences` given to scan. The order of the outputs is the sane
+        `sequences` given to scan. The order of the outputs is the same
         as the order of ``output_info``. For any sequence or output the
-        order of the time slices is the same as the order of the time
-        taps provided. For example if one writes the following :
+        order of the time slices is the same as the one in which they have
+        been given as taps. For example if one writes the following :
 
         .. code-block:: python
 
@@ -129,25 +129,49 @@ def scan( fn
         The list of ``non_sequences`` can also contain shared variables
         used in the function, though ``scan`` is able to figure those
         out on its own so they can be skipped. For the clarity of the
-        code we recommand though to provide them to scan.
+        code we recommand though to provide them to scan. To some extend
+        ``scan`` can also figure out other ``non sequences`` (not shared)
+        even if not passed to scan (but used by `fn`). A simple example of
+        this would be :
+
+        .. code-block:: python
+
+            import theano.tensor as TT
+            W   = TT.matrix()
+            W_2 = W**2
+            def f(x):
+                return TT.dot(x,W_2)
 
         The function is expected to return two things. One is a list of
         outputs ordered in the same order as ``outputs_info``, with the
         difference that there should be only one output variable per
         output initial state (even if no tap value is used). Secondly
-        `fn` should return an update dictionary ( that tells how to
-        update any shared variable after each iteration ste). The
+        `fn` should return an update dictionary (that tells how to
+        update any shared variable after each iteration step). The
         dictionary can optionally be given as a list of tuples. There is
         no constraint on the order of these two list, ``fn`` can return
         either ``(outputs_list, update_dictionary)`` or
         ``(update_dictionary, outputs_list)`` or just one of the two (in
         case the other is empty).
 
+        To use ``scan`` as a while loop, the user needs to change the
+        function ``fn`` such that also a stopping condition is returned.
+        To do so, he/she needs to wrap the condition in an ``until`` class.
+        The condition should be returned as a third element, for example:
+
+        .. code-block:: python
+
+            ...
+            return [y1_t, y2_t], {x:x+1}, theano.scan_module.until(x < 50)
+
+        Note that a number of steps (considered in here as the maximum
+        number of steps ) is still required even though a condition is
+        passed (and it is used to allocate memory if needed). = {}):
 
     :param sequences:
         ``sequences`` is the list of Theano variables or dictionaries
         describing the sequences ``scan`` has to iterate over. If a
-        sequence is given as wrapped in a dictionary a set of optional
+        sequence is given as wrapped in a dictionary, then a set of optional
         information can be provided about the sequence. The dictionary
         should have the following keys:
 
@@ -198,13 +222,6 @@ def scan( fn
           ``fn``. They are provided as a list of *negative* integers,
           where a value ``k`` implies that at iteration step ``t`` scan
           will pass to ``fn`` the slice ``t+k``.
-        * ``return_steps`` -- Integer representing the number of steps
-          to return for the current steps. For example, if ``k`` is
-          provided, ``scan`` will return ``output[-k:]``. This is meant
-          as a hint, based on ``k`` and the past taps of the outputs used,
-          scan can be smart about the amount of memory it requires to
-          store intermidiate results. If not given, or ``0``, ``scan``
-          will return all computed steps.
 
         ``scan`` will follow this logic if partial information is given:
 
@@ -217,12 +234,12 @@ def scan( fn
         * If you wrap an output in a dictionary but you do not provide any
           initial state, it assumes that you are not using any form of
           taps.
-        * If you provide a ``None`` instead of a variable or a dictionary
-          ``scan`` assumes that you will not use any taps for this output
-          (like for example in case of a map)
+        * If you provide a ``None`` instead of a variable or a empty
+          dictionary ``scan`` assumes that you will not use any taps for
+          this output (like for example in case of a map)
 
         If ``outputs_info`` is an empty list or None, ``scan`` assumes
-        that no tap is used for any of the otuputs. If information is
+        that no tap is used for any of the outputs. If information is
         provided just for a subset of the outputs an exception is
         raised (because there is no convention on how scan should map
         the provided information to the outputs of ``fn``)
@@ -230,8 +247,9 @@ def scan( fn
 
     :param non_sequences:
         ``non_sequences`` is the list of arguments that are passed to
-        ``fn`` at each steps. Once can opt to exclude shared variables
-        used in ``fn`` from this list.
+        ``fn`` at each steps. One can opt to exclude variable
+        used in ``fn`` from this list as long as they are part of the
+        computational graph, though for clarity we encourage not to do so.
 
 
     :param n_steps:
@@ -239,10 +257,9 @@ def scan( fn
         or Theano scalar. If any of the input sequences do not have
         enough elements, scan will raise an error. If the *value is 0* the
         outputs will have *0 rows*. If the value is negative, ``scan``
-        run backwards in time. If the ``go_backwards`` flag is already
+        will run backwards in time. If the ``go_backwards`` flag is already
         set and also ``n_steps`` is negative, ``scan`` will run forward
-        in time. If n stpes is not provided, or is a constant that
-        evaluates to ``None``, ``inf`` or ``NaN``, ``scan`` will figure
+        in time. If n stpes is not provided, ``scan`` will figure
         out the amount of steps it should run given its input sequences.
 
 
@@ -264,30 +281,42 @@ def scan( fn
 
 
     :param name:
-        When profiling ``scan`` it is crucial to provide a name for any
+        When profiling ``scan``, it is crucial to provide a name for any
         instance of ``scan``. The profiler will produce an overall
-        profile of your code as well as profiles for doing one iteration
-        step for each instance of ``scan``. The ``name`` of the instance is
-        how you differentiate between all these profiles.
-
+        profile of your code as well as profiles for the computation of
+        one step of each instance of ``scan``. The ``name`` of the instance
+        appears in those profiles and can greatly help to disambiguate
+        information.
 
     :param mode:
         It is recommended to leave this argument to None, especially
         when profiling ``scan`` (otherwise the results are not going to
-        be accurate). If you prefer the computations of one step os
-        ``scan`` to be done differently then the entire function set
-        this parameters (see ``theano.function`` for details about
+        be accurate). If you prefer the computations of one step of
+        ``scan`` to be done differently then the entire function, you
+        can use this parameter to describe how the computations in this
+        loop are done (see ``theano.function`` for details about
         possible values and their meaning).
 
+    :param profile:
+        Flag or string. If true, or different from the empty string, a
+        profile object will be created and attached to the inner graph of
+        scan. In case ``profile`` is True, the profile object will have the
+        name of the scan instance, otherwise it will have the passed string.
+        Profile object collect (and print) information only when running the
+        inner graph with the new cvm linker ( with default modes,
+        other linkers this argument is useless)
 
     :rtype: tuple
     :return: tuple of the form (outputs, updates); ``outputs`` is either a
              Theano variable or a list of Theano variables representing the
              outputs of ``scan`` (in the same order as in
-             ``outputs_info``). ``updates`` is a dictionary specifying the
-             update rules for all shared variables used in the scan
-             operation. This dictionary should be passed to
-             ``theano.function`` when you compile your function.
+             ``outputs_info``). ``updates`` is a subclass of dictionary
+             specifying the
+             update rules for all shared variables used in scan
+             This dictionary should be passed to ``theano.function`` when
+             you compile your function. The change compared to a normal
+             dictionary is that we validate that keys are SharedVariable
+             and addition of those dictionary are validated to be consistent.
     """
     # General observation : this code is executed only once, at creation
     # of the computational graph, so we don't yet need to be smart about
@@ -311,8 +340,15 @@ def scan( fn
 
     seqs      = wrap_into_list(sequences)
     outs_info = wrap_into_list(outputs_info)
-    non_seqs  = wrap_into_list(non_sequences)
 
+    # Make sure we get rid of numpy arrays or ints or anything like that
+    # passed as inputs to scan
+    non_seqs = []
+    for elem in wrap_into_list(non_sequences):
+        if not isinstance(elem, gof.Variable):
+            non_seqs.append(tensor.as_tensor_variable(elem))
+        else:
+            non_seqs.append(elem)
 
     # If we provided a known number of steps ( before compilation)
     # and if that number is 1 or -1, then we can skip the Scan Op,
@@ -325,7 +361,7 @@ def scan( fn
     else:
         try :
             n_fixed_steps = opt.get_constant_value(n_steps)
-        except:
+        except (TypeError, AttributeError):
             n_fixed_steps = None
 
     # Check n_steps is an int
@@ -351,10 +387,16 @@ def scan( fn
 
     # wrap outputs info in a dictionary if they are not already in one
     for i in xrange(n_outs):
-        if outs_info[i]:
+        if outs_info[i] is not None:
             if isinstance(outs_info[i], dict):
+                # DEPRICATED :
                 if outs_info[i].get('return_steps', None):
+                    _logger.warning( ("Using `return_steps` has been depricated."
+                              " Simply select the entries you need using "
+                              " a subtensor. Scan will optimize memory "
+                              " consumption, so do not worry about that."))
                     return_steps[i] = outs_info[i]['return_steps']
+                # END
 
             if not isinstance(outs_info[i], dict):
                 # by default any output has a tap value of -1
@@ -370,10 +412,10 @@ def scan( fn
                 # ^ initial state but taps not provided
                 if outs_info[i].has_key('taps'):
                     # ^ explicitly provided a None for taps
-                    warning (' Output %s ( index %d) has a initial state '
-                             ' but taps is explicitly set to None ' % (
-                                 getattr(outs_info[i]['initial'],'name','None')
-                                 , i) )
+                    _logger.warning('Output %s ( index %d) has a initial state '
+                             'but taps is explicitly set to None ',
+                             getattr(outs_info[i]['initial'],'name','None'),
+                             i)
                 outs_info[i]['taps'] = [-1]
         else:
             # if a None is provided as the output info we replace it
@@ -427,8 +469,8 @@ def scan( fn
                         if config.compute_test_value != 'ignore':
                             # No need to print a warning or raise an error now,
                             # it will be done when fn will be called.
-                            info(('Cannot compute test value for the inner '
-                                'function of scan, input value missing'), e)
+                            _logger.info(('Cannot compute test value for the inner '
+                                'function of scan, input value missing %s'), e)
 
                 # Add names to slices for debugging and pretty printing ..
                 # that is if the input already has a name
@@ -546,6 +588,10 @@ def scan( fn
 
             actual_arg = init_out['initial']
             arg = safe_new(init_out['initial'])
+            if isinstance(arg, tensor.Constant):
+                # safe new returns a clone of the constants, but that is not
+                # what we need for initial states
+                arg = arg.type()
 
             # Try to transfer test_value to the new variable
             if config.compute_test_value != 'off':
@@ -555,8 +601,8 @@ def scan( fn
                     if config.compute_test_value != 'ignore':
                         # No need to print a warning or raise an error now,
                         # it will be done when fn will be called.
-                        info(('Cannot compute test value for the inner '
-                            'function of scan, input value missing'), e)
+                        _logger.info(('Cannot compute test value for the inner '
+                            'function of scan, input value missing %s'), e)
 
             if getattr(init_out['initial'],'name', None) is not None:
                 arg.name = init_out['initial'].name+'[t-1]'
@@ -608,13 +654,13 @@ def scan( fn
                 # Try to transfer test_value to the new variable
                 if config.compute_test_value != 'off':
                     try:
-                        nw_slice.tag.test_value = Op._get_test_value(_init_out_var_slice)
+                        nw_slice.tag.test_value = gof.Op._get_test_value(_init_out_var_slice)
                     except AttributeError, e:
                         if config.compute_test_value != 'ignore':
                             # No need to print a warning or raise an error now,
                             # it will be done when fn will be called.
-                            info(('Cannot compute test value for the inner '
-                                'function of scan, input value missing.'), e)
+                            _logger.info(('Cannot compute test value for the inner '
+                                'function of scan, input value missing. %s'), e)
 
                 # give it a name or debugging and pretty printing
                 if getattr(init_out['initial'],'name', None) is not None:
@@ -669,17 +715,20 @@ def scan( fn
                 ordered_args +
                 non_seqs     )
 
-    # add only the non-shared variables to the arguments of the dummy
-    # function [ a function should not get shared variables as input ]
-    # this could happen if for example the initial state of an output is a
-    # shared variable for which we use only the last step (i.e. no
-    # subtensort is applied to the shared variable )
+    # add only the non-shared variables and non-constants to the arguments of the dummy
+    # function [ a function should not get shared variables or constants as input ]
     dummy_args = [arg for arg in args
-                  if not isinstance(arg, SharedVariable)]
+                  if (not isinstance(arg, SharedVariable) and
+                      not isinstance(arg, tensor.Constant) )]
     # when we apply the lambda expression we get a mixture of update rules
     # and outputs that needs to be separated
 
-    outputs, updates = scan_utils.get_updates_and_outputs(fn(*args))
+
+    condition, outputs, updates = scan_utils.get_updates_and_outputs(fn(*args))
+    if condition is not None:
+        as_while = True
+    else:
+        as_while = False
     ##
     ###   Step 3. Check if we actually need scan and remove it if we don't
     ##
@@ -688,6 +737,10 @@ def scan( fn
     if n_fixed_steps in [1, -1]:
         # We do not need to use the scan op anymore, so we can just return
         # the outputs and updates we have
+        if condition is not None:
+            _logger.warning( ('When the number of steps is fixed and equal to 1,'
+                      ' the provided stopping condition, ', str(condition),
+                      ' is ignored'))
 
         for pos, inner_out in enumerate(outputs):
             # we need to see if we need to pad our sequences with an
@@ -720,21 +773,27 @@ def scan( fn
 
     # extract still missing inputs (there still might be so) and add them
     # as non sequences at the end of our args
-
+    fake_nonseqs = [x.type() for x in non_seqs]
+    fake_outputs = scan_utils.clone(outputs,
+                                    replace=dict(zip(non_seqs,
+                                                     fake_nonseqs)))
     all_inputs = itertools.ifilter(
         lambda x: ( isinstance(x, gof.Variable) and
                    not isinstance(x, SharedVariable) and
                    not isinstance(x, gof.Constant) ),
-        gof.graph.inputs( outputs) )
-    extra_inputs     = filter( lambda x: x not in args,
+        gof.graph.inputs( fake_outputs) )
+    extra_inputs     = filter( lambda x: x not in args + fake_nonseqs,
                                     all_inputs)
     non_seqs += extra_inputs
     ## Note we do not use all_inputs directly since the order of variables
     ## in args is quite important
     dummy_args += extra_inputs
 
+    dummy_outs = outputs
+    if condition is not None:
+        dummy_outs.append(condition)
     dummy_f = function( dummy_args
-                       , outputs
+                       , dummy_outs
                        , updates = updates
                        , mode = compile.mode.Mode(linker='py',
                                                   optimizer=None) )
@@ -752,13 +811,18 @@ def scan( fn
     # assumed outputs until now (provided by the user) there can be
     # only one explanation: No information is provided for any of the
     # outputs (i.e. we are dealing with a map)
-    if not ( len(dummy_f.maker.outputs) == n_outs or outs_info == []):
+    tmp_dummy_f_outs = len(dummy_f.maker.outputs)
+    if as_while:
+        tmp_dummy_f_outs -= 1
+    if not ( tmp_dummy_f_outs == n_outs or outs_info == []):
         raise ValueError('Please provide None as output_info for '
                          'any output that does not feed back into '
                          'scan (i.e. it behaves like a map) ')
 
     if outs_info == []:
         n_outs = len(dummy_f.maker.outputs)
+        if as_while:
+            n_outs = n_outs - 1
         outs_info   = [ dict() for x in xrange(n_outs) ]
 
 
@@ -810,24 +874,20 @@ def scan( fn
     other_inner_args = []
 
     other_scan_args  += [ arg for arg in non_seqs
-                        if not isinstance(arg, SharedVariable) ]
+                        if (not isinstance(arg, SharedVariable) and
+                            not isinstance(arg, tensor.Constant))]
+
+    ## Step 5.6 all shared variables with no update rules
+    other_inner_args += [ safe_new(arg,'_copy') for arg in non_seqs
+                         if (not isinstance(arg, SharedVariable) and
+                             not isinstance(arg, tensor.Constant))]
 
-    ## Step 5.6 all non sequences including shared variables with no update rules
-    def new_variable( v ):
-        if isinstance(new_variable, tensor.Constant):
-            return v.clone()
-        new_v = safe_new(v)
-        if getattr(v,'name', None) is not None:
-            new_v.name = v.name + '_copy'
-        return new_v
-    other_inner_args += [ new_variable(arg) for arg in non_seqs
-                         if not isinstance(arg, SharedVariable) ]
     givens.update( dict( zip(other_scan_args, other_inner_args) ))
     other_shared_scan_args  = [ arg.variable for arg
                         in dummy_f.maker.expanded_inputs
                         if ( isinstance(arg.variable, SharedVariable) and
                             not arg.update) ]
-    other_shared_inner_args = [ new_variable(arg.variable) for arg
+    other_shared_inner_args = [ safe_new(arg.variable, '_copy') for arg
                         in dummy_f.maker.expanded_inputs
                         if ( isinstance(arg.variable, SharedVariable) and
                             not arg.update) ]
@@ -852,6 +912,8 @@ def scan( fn
                    sit_sot_inner_outputs +
                    nit_sot_inner_outputs +
                    shared_inner_outputs  )
+    if condition is not None:
+        inner_outs.append(condition)
     if cuda.cuda_available:
         # very often we end up in this situation when we want to
         # replace w with w_copy, where w is CudaNdarray
@@ -893,13 +955,15 @@ def scan( fn
     info['mode']               = mode
     info['inplace']            = False
     info['gpu']                = False
+    info['as_while']           = as_while
+    info['profile']            = profile
 
     local_op = scan_op.Scan( inner_inputs, new_outs, info )
 
     ##
     ### Step 8. Compute the outputs using the scan op
     ##
-    scan_inputs = ( scan_seqs                                    +
+    _scan_inputs = ( scan_seqs                                   +
                    mit_mot_scan_inputs                           +
                    mit_sot_scan_inputs                           +
                    sit_sot_scan_inputs                           +
@@ -908,7 +972,15 @@ def scan( fn
                    other_shared_scan_args                        +
                    other_scan_args                               )
 
-    scan_inputs = [safe_to_cpu(x) for x in ([actual_n_steps] + scan_inputs)]
+    scan_inputs = []
+    for arg in [actual_n_steps]+ _scan_inputs:
+        try:
+            arg = tensor.as_tensor_variable(arg)
+        except TypeError:
+            # This happens for Random States for e.g. but it is a good way
+            # to make sure no input is a cuda ndarrays
+            pass
+        scan_inputs += [arg]
     scan_outs = local_op(* scan_inputs  )
     if type(scan_outs) not in (list,tuple):
         scan_outs = [scan_outs]
@@ -917,7 +989,7 @@ def scan( fn
     ###         and so on ...
     ##
 
-    update_map = {}
+    update_map = Updates()
     def remove_dimensions( outs, steps_return, offsets = None):
         out_ls = []
         for idx, out in enumerate(outs):
